@@ -57,6 +57,8 @@ extern "C" {
 struct lttng_channel;
 struct lttng_session;
 struct lttng_ust_lib_ring_buffer_ctx;
+struct lttng_ust_context_app;
+struct lttng_event_field;
 
 /*
  * Data structures used by tracepoint event declarations, and by the
@@ -86,6 +88,8 @@ enum lttng_abstract_types {
 	atype_sequence,
 	atype_string,
 	atype_float,
+	atype_dynamic,
+	atype_struct,
 	NR_ABSTRACT_TYPES,
 };
 
@@ -97,11 +101,25 @@ enum lttng_string_encodings {
 	NR_STRING_ENCODINGS,
 };
 
+struct lttng_enum_value {
+	unsigned long long value;
+	unsigned int signedness:1;
+};
+
+enum lttng_enum_entry_options {
+	LTTNG_ENUM_ENTRY_OPTION_IS_AUTO = 1U << 0,
+};
+
 #define LTTNG_UST_ENUM_ENTRY_PADDING	16
 struct lttng_enum_entry {
-	unsigned long long start, end;	/* start and end are inclusive */
+	struct lttng_enum_value start, end; /* start and end are inclusive */
 	const char *string;
-	char padding[LTTNG_UST_ENUM_ENTRY_PADDING];
+	union {
+		struct {
+			unsigned int options;
+		} LTTNG_PACKED extra;
+		char padding[LTTNG_UST_ENUM_ENTRY_PADDING];
+	} u;
 };
 
 #define __type_integer(_type, _byte_order, _base, _encoding)	\
@@ -176,7 +194,8 @@ struct lttng_float_type {
 union _lttng_basic_type {
 	struct lttng_integer_type integer;
 	struct {
-		const char *name;
+		const struct lttng_enum_desc *desc;	/* Enumeration mapping */
+		struct lttng_integer_type container_type;
 	} enumeration;
 	struct {
 		enum lttng_string_encodings encoding;
@@ -205,16 +224,19 @@ struct lttng_type {
 			struct lttng_basic_type length_type;
 			struct lttng_basic_type elem_type;
 		} sequence;
+		struct {
+			uint32_t nr_fields;
+			struct lttng_event_field *fields;	/* Array of fields. */
+		} _struct;
 		char padding[LTTNG_UST_TYPE_PADDING];
 	} u;
 };
 
 #define LTTNG_UST_ENUM_TYPE_PADDING	24
-struct lttng_enum {
+struct lttng_enum_desc {
 	const char *name;
-	struct lttng_type container_type;
 	const struct lttng_enum_entry *entries;
-	unsigned int len;
+	unsigned int nr_entries;
 	char padding[LTTNG_UST_ENUM_TYPE_PADDING];
 };
 
@@ -234,10 +256,29 @@ struct lttng_event_field {
 	char padding[LTTNG_UST_EVENT_FIELD_PADDING];
 };
 
-union lttng_ctx_value {
-	int64_t s64;
-	const char *str;
-	double d;
+enum lttng_ust_dynamic_type {
+	LTTNG_UST_DYNAMIC_TYPE_NONE,
+	LTTNG_UST_DYNAMIC_TYPE_S8,
+	LTTNG_UST_DYNAMIC_TYPE_S16,
+	LTTNG_UST_DYNAMIC_TYPE_S32,
+	LTTNG_UST_DYNAMIC_TYPE_S64,
+	LTTNG_UST_DYNAMIC_TYPE_U8,
+	LTTNG_UST_DYNAMIC_TYPE_U16,
+	LTTNG_UST_DYNAMIC_TYPE_U32,
+	LTTNG_UST_DYNAMIC_TYPE_U64,
+	LTTNG_UST_DYNAMIC_TYPE_FLOAT,
+	LTTNG_UST_DYNAMIC_TYPE_DOUBLE,
+	LTTNG_UST_DYNAMIC_TYPE_STRING,
+	_NR_LTTNG_UST_DYNAMIC_TYPES,
+};
+
+struct lttng_ctx_value {
+	enum lttng_ust_dynamic_type sel;
+	union {
+		int64_t s64;
+		const char *str;
+		double d;
+	} u;
 };
 
 struct lttng_perf_counter_field;
@@ -245,17 +286,18 @@ struct lttng_perf_counter_field;
 #define LTTNG_UST_CTX_FIELD_PADDING	40
 struct lttng_ctx_field {
 	struct lttng_event_field event_field;
-	size_t (*get_size)(size_t offset);
+	size_t (*get_size)(struct lttng_ctx_field *field, size_t offset);
 	void (*record)(struct lttng_ctx_field *field,
 		       struct lttng_ust_lib_ring_buffer_ctx *ctx,
 		       struct lttng_channel *chan);
 	void (*get_value)(struct lttng_ctx_field *field,
-			 union lttng_ctx_value *value);
+			 struct lttng_ctx_value *value);
 	union {
 		struct lttng_perf_counter_field *perf_counter;
 		char padding[LTTNG_UST_CTX_FIELD_PADDING];
 	} u;
 	void (*destroy)(struct lttng_ctx_field *field);
+	char *field_name;	/* Has ownership, dynamically allocated. */
 };
 
 #define LTTNG_UST_CTX_PADDING	20
@@ -300,7 +342,7 @@ struct lttng_probe_desc {
 /* Data structures used by the tracer. */
 
 enum lttng_enabler_type {
-	LTTNG_ENABLER_WILDCARD,
+	LTTNG_ENABLER_STAR_GLOB,
 	LTTNG_ENABLER_EVENT,
 };
 
@@ -380,6 +422,8 @@ struct lttng_bytecode_runtime {
 	uint64_t (*filter)(void *filter_data, const char *filter_stack_data);
 	int link_failed;
 	struct cds_list_head node;	/* list of bytecode runtime in event */
+	struct lttng_session *session;
+	struct lttng_event *event;
 };
 
 /*
@@ -407,8 +451,6 @@ struct lttng_event {
 	void *_deprecated1;
 	struct lttng_ctx *ctx;
 	enum lttng_ust_instrumentation instrumentation;
-	union {
-	} u;
 	struct cds_list_head node;		/* Event list in session */
 	struct cds_list_head _deprecated2;
 	void *_deprecated3;
@@ -424,6 +466,14 @@ struct lttng_event {
 	int registered;			/* has reg'd tracepoint probe */
 };
 
+struct lttng_enum {
+	const struct lttng_enum_desc *desc;
+	struct lttng_session *session;
+	struct cds_list_head node;	/* Enum list in session */
+	struct cds_hlist_node hlist;	/* Session ht of enums */
+	uint64_t id;			/* Enumeration ID in sessiond */
+};
+
 struct channel;
 struct lttng_ust_shm_handle;
 
@@ -439,7 +489,9 @@ struct lttng_channel_ops {
 			unsigned int switch_timer_interval,
 			unsigned int read_timer_interval,
 			unsigned char *uuid,
-			uint32_t chan_id);
+			uint32_t chan_id,
+			const int *stream_fds, int nr_stream_fds,
+			int64_t blocking_timeout);
 	void (*channel_destroy)(struct lttng_channel *chan);
 	union {
 		void *_deprecated1;
@@ -500,6 +552,14 @@ struct lttng_channel {
 	int tstate:1;			/* Transient enable state */
 };
 
+#define LTTNG_UST_STACK_CTX_PADDING	32
+struct lttng_stack_ctx {
+	struct lttng_event *event;
+	struct lttng_ctx *chan_ctx;	/* RCU dereferenced. */
+	struct lttng_ctx *event_ctx;	/* RCU dereferenced. */
+	char padding[LTTNG_UST_STACK_CTX_PADDING];
+};
+
 #define LTTNG_UST_EVENT_HT_BITS		12
 #define LTTNG_UST_EVENT_HT_SIZE		(1U << LTTNG_UST_EVENT_HT_BITS)
 
@@ -507,6 +567,13 @@ struct lttng_ust_event_ht {
 	struct cds_hlist_head table[LTTNG_UST_EVENT_HT_SIZE];
 };
 
+#define LTTNG_UST_ENUM_HT_BITS		12
+#define LTTNG_UST_ENUM_HT_SIZE		(1U << LTTNG_UST_ENUM_HT_BITS)
+
+struct lttng_ust_enum_ht {
+	struct cds_hlist_head table[LTTNG_UST_ENUM_HT_SIZE];
+};
+
 /*
  * IMPORTANT: this structure is part of the ABI between the probe and
  * UST. Fields need to be only added at the end, never reordered, never
@@ -533,6 +600,11 @@ struct lttng_session {
 
 	/* New UST 2.4 */
 	int statedump_pending:1;
+
+	/* New UST 2.8 */
+	struct lttng_ust_enum_ht enums_ht;	/* ht of enumerations */
+	struct cds_list_head enums_head;
+	struct lttng_ctx *ctx;			/* contexts for filters. */
 };
 
 struct lttng_transport {
@@ -545,6 +617,7 @@ struct lttng_transport {
 struct lttng_session *lttng_session_create(void);
 int lttng_session_enable(struct lttng_session *session);
 int lttng_session_disable(struct lttng_session *session);
+int lttng_session_statedump(struct lttng_session *session);
 void lttng_session_destroy(struct lttng_session *session);
 
 struct lttng_channel *lttng_channel_create(struct lttng_session *session,
@@ -573,10 +646,9 @@ int lttng_enabler_attach_exclusion(struct lttng_enabler *enabler,
 		struct lttng_ust_excluder_node *excluder);
 
 int lttng_attach_context(struct lttng_ust_context *context_param,
+		union ust_args *uargs,
 		struct lttng_ctx **ctx, struct lttng_session *session);
-void lttng_context_init(void);
-void lttng_context_exit(void);
-extern struct lttng_ctx *lttng_static_ctx;	/* Used by filtering */
+int lttng_session_context_init(struct lttng_ctx **ctx);
 
 void lttng_transport_register(struct lttng_transport *transport);
 void lttng_transport_unregister(struct lttng_transport *transport);
@@ -585,6 +657,7 @@ void synchronize_trace(void);
 
 int lttng_probe_register(struct lttng_probe_desc *desc);
 void lttng_probe_unregister(struct lttng_probe_desc *desc);
+void lttng_probe_provider_unregister_events(struct lttng_probe_desc *desc);
 int lttng_fix_pending_events(void);
 int lttng_probes_init(void);
 void lttng_probes_exit(void);
@@ -600,8 +673,11 @@ int lttng_add_vpid_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_pthread_id_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_procname_to_ctx(struct lttng_ctx **ctx);
 int lttng_add_ip_to_ctx(struct lttng_ctx **ctx);
+int lttng_add_cpu_id_to_ctx(struct lttng_ctx **ctx);
+int lttng_add_dyntest_to_ctx(struct lttng_ctx **ctx);
 void lttng_context_vtid_reset(void);
 void lttng_context_vpid_reset(void);
+void lttng_context_procname_reset(void);
 
 #ifdef LTTNG_UST_HAVE_PERF_EVENT
 int lttng_add_perf_counter_to_ctx(uint32_t type,
@@ -658,6 +734,17 @@ typedef int (*t_statedump_func_ptr)(struct lttng_session *session);
 void lttng_handle_pending_statedump(void *owner);
 struct cds_list_head *_lttng_get_sessions(void);
 
+struct lttng_enum *lttng_ust_enum_get_from_desc(struct lttng_session *session,
+		const struct lttng_enum_desc *enum_desc);
+
+void lttng_ust_dl_update(void *ip);
+void lttng_ust_fixup_fd_tracker_tls(void);
+
+/* For backward compatibility. Leave those exported symbols in place. */
+extern struct lttng_ctx *lttng_static_ctx;
+void lttng_context_init(void);
+void lttng_context_exit(void);
+
 #ifdef __cplusplus
 }
 #endif
